# Date::Simple - a simple date object
# Modified to always load xs.pm

package Date::Simple;

$VERSION = '2.03';
use Exporter ();
@ISA = ('Exporter');
@EXPORT_OK = ('today', 'ymd', 'd8', 'date', 'leap_year', 'days_in_month');
%EXPORT_TAGS = ('all' => \@EXPORT_OK);

require xs;

use strict;
use Carp ();
use overload
    '+'   => '_add',
    '-'   => '_subtract',
    '<=>' => '_compare',
    'cmp' => '_compare',
    'bool' => sub { 1 },
    '""'  => '_stringify';

sub today {
    my ($y, $m, $d) = (localtime) [5, 4, 3];
    $y += 1900;
    $m += 1;
    return ymd ($y, $m, $d);
}

sub _inval {
    Carp::croak ("Invalid ".(ref($_[0])||$_[0])." constructor args: ('"
		 .join("', '", @_[1..$#_])."')");
}

sub new {
    my ($that, @ymd) = @_;
    my $class = ref($that) || $that;

    if (@ymd == 1) {
	my $x = $ymd[0];
	if(ref $x eq 'ARRAY') {
	    @ymd = @$x;
	}
	elsif (UNIVERSAL::isa ($x, $class)) {
	    return ($x);
	}
	else {
	    @ymd = $x =~ /^(\d{4})-(\d{2})-(\d{2})$/
		or Carp::croak ("'$x' is not a valid ISO formated date");
	}
    }
    if (@ymd == 0) {
	return (today());
    }
    if (@ymd == 3) {
	my $days = ymd_to_days (@ymd);
	return undef if ! defined ($days);
	return (bless (\$days, $class));
    }
    _inval ($class, @ymd);
}

# Call new, but return undef instead of raising an exception.
sub date {
    local ($@);
    return (scalar eval {new (__PACKAGE__, @_)});
}

sub next { return ($_[0] + 1); }
sub prev { return ($_[0] - 1); }

sub _gmtime {
    my ($y, $m, $d) = days_to_ymd (${$_[0]});
    $y -= 1900;
    $m -= 1;
    return (0, 0, 0, $d, $m, $y);
}

sub format {
    my ($self, $format) = @_;
    return "$self" unless defined ($format);
    require POSIX;
    local $ENV{TZ} = 'UTC+0';
    return POSIX::strftime ($format, _gmtime ($self));
}

sub strftime { &format }

1;

